@@ -6,7 +6,7 @@
 #define NEED_EVENTS
 #include <X11/X.h>
 #include "windowstr.h"
-#include <X11/extensions/XKBsrv.h>
+#include <xkbsrv.h>
 
 int
 XkbDDXPrivate(DeviceIntPtr dev,KeyCode key,XkbAction *act)
